<?php

namespace App\Http\Controllers\Api;


use Illuminate\Http\Request;
use App\Http\Resources\DefaultResource;
use App\Http\Controllers\MainApiController;
use App\Http\Requests\VerifyActivationCodeRequest;
use App\Http\Requests\Password\{ResetPassword, SetNewPasswordRequest, ChangePasswordRequest};
use App\Http\Requests\User\UserProfileUpdateRequest;
use App\Http\Requests\User\UserRegisterRequest;
use App\Http\Requests\User\UserLoginRequest;
use App\Http\Requests\User\UserSetPasswordRequest;
use App\Models\User;
use App\Models\PasswordReset;
use App\Services\Common\UserService;
use Illuminate\Support\Facades\DB;
use App\Http\Resources\Api\UserResource;



class AuthController extends MainApiController
{
    private $userService;

    public function __construct(UserService $userService)
    {
        parent::__construct();
        $this->userService = $userService;
    }


    public function update(UserProfileUpdateRequest $request)
    {
        auth()->user()->first_name = $request->first_name;
        auth()->user()->last_name = $request->last_name;
        auth()->user()->phone = $request->phone;
        auth()->user()->dialing_code = $request->dialing_code;
        auth()->user()->country_code = $request->country_code;


        if ($request->has('photo')) {

            $saveFile = $this->saveFile($request->photo, User::IMAGES_DIRECTORY, 'local');
            auth()->user()->photo = $saveFile['fileName'];
        }

        auth()->user()->save();

        return response()->json(['status' => true, 'message' => 'Profile Updated Successfully', 'data' => new UserResource(auth()->user())]);
    }


    public function updatePassword(ChangePasswordRequest $request)
    {
        $user = $this->userService->updatePassword($request, auth()->user()->id);
        return $this->response->success(
            new DefaultResource(['message' => 'Password changed successfully!'])
        );
    }

    public function signup(UserRegisterRequest $request)
    {

        $user = $this->userService->create($request); // defualt type is user

        return response()->json($user);
    }

    public function login(UserLoginRequest $request)
    {

        $data = $this->userService->checkLogin($request, $request->source);

        return response()->json($data);
    }

    public function reset(ResetPassword $request)
    {
        return $this->userService->reset($request);
    }

    public function setPassword(UserSetPasswordRequest $request)
    {
        return $this->userService->verifyAndUpdatePassword($request);
    }

    public function verifyCode(VerifyActivationCodeRequest $request)
    {
        $data = PasswordReset::where('token', $request->token)
            ->where('email', $request->email)
            ->get();
        if (!count($data)) {
            return response()->json(['status' => false, 'message' => 'Invalid token'], 401);
        } else {
            return response()->json(['success' => true, 'message' => 'Code verified', 'token' => $request->token]);
        }
    }

    public function sendVerificationCode(ResetPassword $request)
    {

        $user = $this->userService->getUserByEmail($request->email);
        return $this->userService->sendAccountVerification($user);
    }

    public function logout(Request $request)
    {

        if (auth('sanctum')->check()) {
            auth('sanctum')->user()->currentAccessToken()->delete();
            return response()->json(['status' => true, 'message' => 'Logout Successful']);
        } else {
            return response()->json(['status' => false, 'message' => 'Invalid request'], 401);
        }
    }

    public function profile(Request $request)
    {

        $user = auth()->guard()->user(); //->load(['education', 'skills', 'experience']);
        return response()->json(['status' => true, 'data' => new UserResource($user)]);
    }
}
