<?php

namespace App\Http\Controllers;

use App\Http\Controllers\Controller;
use App\Services\JsonResponseService;
use Symfony\Component\HttpFoundation\Response;

/**
 * Class MainController
 * @package App\Http\Controllers
 */
class MainApiController extends Controller
{
    /**
     * @var \App\Services\JsonResponseService
     */
    protected $response;

    /**
     * number of pagination
     * @var int
     */
    protected $pagination;

    /**
     * MainController constructor.
     */
    public function __construct()
    {
        $this->response = new JsonResponseService;
        $requestPerPage = request('per_page', config('app.pagination'));
        app()->setLocale(request('lang') ? request('lang') : 'en');
        $this->pagination = $requestPerPage > 100 ? 100 : $requestPerPage;
    }

    /**
     * @return \Illuminate\Http\JsonResponse
     */
    public function __invoke()
    {
        return $this->response->fail([], 400, Response::HTTP_NOT_FOUND);
    }

    public function saveFile($file, String $subFolder, String $disk = 'local')
    {
        $extension = $file->getClientOriginalExtension();
        $fileName = time() . rand(1000,9999) . '.' . $extension;
        $fileSize = $file->getSize();
        $fileType = $file->getMimeType();

        $sanitizedName = str_replace($extension, '', \Illuminate\Support\Str::slug($file->getClientOriginalName()));

        if ($disk == "s3") {
            $path = $subFolder . '/' . $fileName . '.' . $extension;
            \Illuminate\Support\Facades\Storage::disk('s3')->put($path, file_get_contents($file));
        } else {        
            $file->move(public_path($subFolder), $fileName);
            $path = "{$subFolder}/{$fileName}";
        }

        return [
            "fileName" => $fileName,
            "fileSize" => $fileSize,
            "fileType" => $fileType,
            "path" => $path,
            "disk" => $disk
        ];
    }
}
