<?php

namespace App\Http\Controllers;

use App\Http\Controllers\MainApiController;
use App\Http\Resources\NotificationResource;
use Illuminate\Http\Request;

class NotificationController extends MainApiController
{
    public function index(Request $request)
    {
        $user = auth()->user();
        $notificationsQuery = $user->notifications();

        // Check if status parameter is provided
        if ($request->has('status')) {
            $status = $request->status;

            // Filter notifications based on status
            if ($status === 'read') {
                // Show only read notifications (where read_at is not null)
                $notificationsQuery->whereNotNull('read_at');
            } elseif ($status === 'unread') {
                // Show only unread notifications (where read_at is null)
                $notificationsQuery->whereNull('read_at');
            }
        }

        // Paginate the filtered notifications
        $notifications = $notificationsQuery->paginate();

        return NotificationResource::collection($notifications);
        // return $notifications;
    }

    public function mark(Request $request)
    {
        auth()->user()->unreadNotifications()->get()->map(function ($n) {
            $n->markAsRead();
        });

        return response()->json(["message" => "Marked unread all notification"], 200);
    }

    public function unread()
    {
        return NotificationResource::collection(
            auth()->user()->unreadNotifications()->paginate()
        );
    }

    public function markAsUnread(Request $request)
    {
        $notification = auth()->user()->notifications()->where('id', $request->uuid)->get();
        return $notification->markAsUnread();
    }

    public function individual(Request $request)
    {
        $notification = auth()->user()->notifications()->where('id', $request->uuid)->get();
        return $notification->markAsRead();
        return NotificationResource::collection(
            $notification
        );
    }

    public function unreadNotificationCount()
    {

        return response()->json(["count" => auth()->user()->unreadNotifications()->count()], 200);
    }

    public function unreadMessageCount()
    {
        return response()->json(["count" => auth()->user()->unreadNotifications()->whereRaw('JSON_CONTAINS(`data`, \'{"data": {"type": "message"}}\')')->count()], 200);
    }
}
