<?php

namespace App\Http\Resources;

use Illuminate\Http\Request;
use Illuminate\Http\Resources\Json\JsonResource;
use Carbon\Carbon;
use Illuminate\Support\Str;
class NotificationResource extends JsonResource
{
    /**
     * Transform the resource into an array.
     *
     * @return array<string, mixed>
     */
    public function toArray(Request $request): array
    {

        $createdAt = Carbon::parse($this->created_at);
        $now = Carbon::now();
        $diffInMinutes = $now->diffInMinutes($createdAt);

        if ($diffInMinutes < 1) {
            // Less than a minute
            $timeAgo = $now->diffInSeconds($createdAt) . ' seconds ago';
        } elseif ($diffInMinutes < 60) {
            // Less than an hour
            $timeAgo = $diffInMinutes . ' minutes ago';
        } elseif ($diffInMinutes < 1440) {
            // Less than 24 hours
            $timeAgo = $createdAt->diffInHours($now) . ' hours ago';
        } else {
            // More than 24 hours
            $timeAgo = $createdAt->diffInDays($now) . ' days ago';
        }

        $limitedTitle = Str::limit($this->data['title'], 25);

        return [
            "id" => $this->id,
            "title" => $limitedTitle,
            'message' => $this->data['message'],
            "type" => $this->type,
            "content_id" => $this->notifiable_id,
            "created_at" => $this->created_at,
            'date' => $this->created_at->format('M d, Y'),
            'read' => $this->read_at,
            'time' => $timeAgo
        ];
    }
}
