<?php

namespace App\Models;

// use Illuminate\Contracts\Auth\MustVerifyEmail;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Foundation\Auth\User as Authenticatable;
use Illuminate\Notifications\Notifiable;
use Laravel\Sanctum\HasApiTokens;

class User extends Authenticatable
{
    use HasApiTokens, HasFactory, Notifiable;

    /**
     * The attributes that are mass assignable.
     *
     * @var array<int, string>
     */
    protected $guarded = ['id'];

    /**
     * The attributes that should be hidden for serialization.
     *
     * @var array<int, string>
     */
    protected $hidden = [
        'password',
        'remember_token',
    ];

    /**
     * The attributes that should be cast.
     *
     * @var array<string, string>
     */
    protected $casts = [
        'email_verified_at' => 'datetime',
        'password' => 'hashed',
    ];

    protected $appends = ['full_photo_url'];


    public const IMAGES_DIRECTORY = 'profile/img/';
    public const PENDING_STATUS = 0;
    public const ACTIVE_STATUS = 1;
    public const INACTIVE_STATUS = 2;
    public const REJECT_STATUS = 3;


    public function vendor()
    {
        return $this->hasOne(Vendor::class);
    }

    public function stores()
    {
        return $this->hasMany(Store::class);
    }

    public function specials()
    {
        return $this->hasMany(Special::class);
    }

    public function testingRequests()
    {
        return $this->hasMany(TestingRequest::class);
    }

    public function feedbacks()
    {
        return $this->hasMany(Feedback::class);
    }

    public function orders()
    {
        return $this->hasMany(Order::class);
    }

    public function reviews()
    {
        return $this->hasMany(Review::class);
    }

    public function wishlists()
    {
        return $this->hasMany(Wishlist::class);
    }

    public function reason()
    {
        return $this->morphOne(Reason::class, 'reasonable');
    }

    public function payemntLogs()
    {
        return $this->hasMany(PaymentLog::class);
    }

    public function getFullPhotoUrlAttribute()
    {
        return $this->photo ? asset(User::IMAGES_DIRECTORY . $this->photo) : asset(User::IMAGES_DIRECTORY . 'default.png');
    }
}
