<?php


namespace App\Services\Admin;

use App\Models\User;
use App\Services\Common\CrudService;

class AdminUserService extends CrudService
{
    protected $model;

    public function __construct()
    {
        $this->model = new User();
    }

    public function getAllUsers($pagination = null)
    {
        $pagination = $pagination ?? request()->input('per_page', 10);
        $users = User::where('type', '=', request("type"))
            ->when(request()->filled('status'), function ($q) {
                $q->whereStatus(request('status'));
            })
            ->when(request()->filled('search'), function ($q) {
                $q->where(function ($q) {
                    $q->where('first_name', 'like', '%' . request("search") . '%')
                        ->orWhere('last_name', 'like', '%' . request("search") . '%')
                        ->orWhere('email', 'like', '%' . request("search") . '%');
                });
            })
            ->when(request()->filled('from'), function ($q) {
                $q->whereDate('created_at', '>=', request('from'));
            })
            ->when(request()->filled('to'), function ($q) {
                $q->whereDate('created_at', '<=', request('to'));
            })
            ->orderBy('id', 'desc')
            ->paginate($pagination);

        return $users;
    }

    public function switchStatus($user)
    {
        $user->status = ($user->status == 0) ? 1 : 0;
        $user->save();

        return $user;
    }

    public function show($user)
    {
        $user = User::where('id', $user->id)->get();
        return $user;
    }
}
