<?php


namespace App\Services;

use Hash;
use App\Models\User;
use App\Models\Order;
use App\Models\Feedback;
use HelperConstants;
use App\Services\Common\UserService;
use App\Models\PushNotification;


class AdminService extends UserService
{

    public function __construct()
    {
        $this->model = new User;
    }

    public function getPushNotifications($pagination)
    {
        $notifications = PushNotification::when(request()->filled('user_type'), function ($q) {
            $q->where('user_type', request('user_type'));
        })->when(request()->filled('search'), function ($q) {
            $q->where(function ($q) {
                $q->where('title', 'like', '%' . request("search") . '%')
                    ->orWhere('message', 'like', '%' . request("search") . '%');
            });
        })
            ->when(request()->filled('from'), function ($q) {
                $q->whereDate('created_at', '>=', request('from'));
            })
            ->when(request()->filled('to'), function ($q) {
                $q->whereDate('created_at', '<=', request('to'));
            })
            ->orderBy('id', 'desc')
            ->paginate($pagination);

        return $notifications;
    }

    public function createPushNotification($request)
    {
        $push_notification = new PushNotification();
        $push_notification->user_type = $request->user_type;
        $push_notification->title = $request->title;
        $push_notification->message = $request->message;
        $push_notification->save();

        return $push_notification;
    }

    public function stats()
    {
        $monthlyTrend = User::selectRaw('COUNT(id) as count')->whereRaw("created_at LIKE '" . date('Y-m') . "%'")->where('type', '!=', 'admin')->first();
        $totalUsers = User::where('type', 'user')->orWhere('type', '!=', 'admin')->count();
        $monthlyTrend = (isset($monthlyTrend->count) && $monthlyTrend->count > 0 ? round((($monthlyTrend->count / $totalUsers) * 100), 2) : 0);

        $monthlyTrendOrder = Order::selectRaw('COUNT(id) as count')->whereRaw("created_at LIKE '" . date('Y-m') . "%'")->first();
        $totalOrders = Order::count();
        $monthlyTrendOrder = (isset($monthlyTrendOrder->count) && $monthlyTrendOrder->count > 0 ? round((($monthlyTrendOrder->count / $totalOrders) * 100), 2) : 0);

        $monthlyTrendFeedback = Feedback::selectRaw('COUNT(id) as count')->whereRaw("created_at LIKE '" . date('Y-m') . "%'")->first();
        $totalFeedbacks = Feedback::count();
        $monthlyTrendFeedback = (isset($monthlyTrendFeedback->count) && $monthlyTrendFeedback->count > 0 ? round((($monthlyTrendFeedback->count / $totalFeedbacks) * 100), 2) : 0);

        return response()->json([
            'status' => true,
            'total_users' => $totalUsers,
            'user_trend' => $monthlyTrend,
            'total_orders' => $totalOrders,
            'order_trend' => $monthlyTrendOrder,
            'total_feedbacks' => $totalFeedbacks,
            'feedback_trend' => $monthlyTrendFeedback,
        ]);
    }

    public function graphData($request)
    {
        $year = $request->year;
        $jan = Order::whereRaw("created_at LIKE '" . date($year . '-01') . "%'")->sum('total');
        $feb = Order::whereRaw("created_at LIKE '" . date($year . '-02') . "%'")->sum('total');
        $mar = Order::whereRaw("created_at LIKE '" . date($year . '-03') . "%'")->sum('total');
        $apr = Order::whereRaw("created_at LIKE '" . date($year . '-04') . "%'")->sum('total');
        $may = Order::whereRaw("created_at LIKE '" . date($year . '-05') . "%'")->sum('total');
        $jun = Order::whereRaw("created_at LIKE '" . date($year . '-06') . "%'")->sum('total');
        $jul = Order::whereRaw("created_at LIKE '" . date($year . '-07') . "%'")->sum('total');
        $aug = Order::whereRaw("created_at LIKE '" . date($year . '-08') . "%'")->sum('total');
        $sep = Order::whereRaw("created_at LIKE '" . date($year . '-09') . "%'")->sum('total');
        $oct = Order::whereRaw("created_at LIKE '" . date($year . '-10') . "%'")->sum('total');
        $nov = Order::whereRaw("created_at LIKE '" . date($year . '-11') . "%'")->sum('total');
        $dec = Order::whereRaw("created_at LIKE '" . date($year . '-12') . "%'")->sum('total');

        return response()->json([
            'status' => true,
            'jan' => $jan,
            'feb' => $feb,
            'mar' => $mar,
            'apr' => $apr,
            'may' => $may,
            'jun' => $jun,
            'jul' => $jul,
            'aug' => $aug,
            'sep' => $sep,
            'oct' => $oct,
            'nov' => $nov,
            'dec' => $dec,
        ]);
    }
}
