<?php

namespace App\Services\Common;

use App\Models\Role;
use App\Models\User;
use App\Models\Notification;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Auth;
use App\Core\Notifications\PushNotification;
use Carbon\Carbon;
use Illuminate\Http\Request;


class NotificationService
{


    // public function notifyAdmins($data)
    // {
    //     try {
    //         Log::debug("notifyAdmins 1: Get admins");
    //         $admins = User::where("type", "admin")->get();
    //         if (count($admins) > 0) {
    //             Log::debug("notifyAdmins 2: notify to admins");


    //             foreach ($admins as $key => $admin) {


    //                 $admin->notify(new PushNotification(
    //                     $data["title"],
    //                     $data["body"],
    //                     $data["data"],

    //                 ));
    //             }
    //             Log::debug("notifyAdmins 3: Notification sent successfully");
    //             return "Notification sent successfully";
    //         }
    //         Log::debug("notifyAdmins 4: Admin not found");
    //         return "Admin not found";
    //     } catch (\Exception $e) {
    //         throw new \Exception($e->getMessage());
    //     }
    // }

    // public function notifyUser($userId, $data)
    // {
    //     try {
    //         Log::debug("notifyUser 1: Get user");
    //         $user = User::find($userId);  // Find user by ID

    //         if ($user) {
    //             Log::debug("notifyUser 2: notify to user");

    //             // Send notification to the single user
    //             $user->notify(new PushNotification(
    //                 $data["title"],
    //                 $data["body"]
    //             ));

    //             Log::debug("notifyUser 3: Notification sent successfully");
    //             return "Notification sent successfully to user";
    //         }

    //         Log::debug("notifyUser 4: User not found");
    //         return "User not found";
    //     } catch (\Exception $e) {
    //         throw new \Exception($e->getMessage());
    //     }
    // }

    public function getUnreadNotifications()
    {
        try {
            $user = Auth::user();

            if (!$user) {
                return ["bell" => collect(), "count" => 0]; // Return empty collection and count if no user is logged in
            }

            // Accessing notifications directly through the user model
            $notifications = $user->notifications()
                ->whereNull('read_at')
                ->orderBy('created_at', 'desc')
                ->get();

            // Format the created_at timestamp to "time ago" format
            foreach ($notifications as $notification) {
                $notification->created_at_human = Carbon::parse($notification->created_at)->diffForHumans();
            }

            return [
                "bell" => $notifications,
                "count" => $notifications->count()
            ];
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }
    }

    public function getAllNotifications(Request $request)
    {
        try {
            $user = Auth::user();

            if (!$user) {
                return []; // Return empty collection if no user is logged in
            }

            // Accessing notifications directly through the user model
            $notificationsQuery = $user->notifications()
                ->orderBy('created_at', 'desc');

            // Check for the status filter
            if ($request->has('status')) {
                $status = $request->status;

                // Filter notifications based on status
                if ($status === 'read') {
                    // Show only read notifications (where read_at is not null)
                    $notificationsQuery->whereNotNull('read_at');
                } elseif ($status === 'unread') {
                    // Show only unread notifications (where read_at is null)
                    $notificationsQuery->whereNull('read_at');
                }
            }

            // Paginate the results
            $notifications = $notificationsQuery->paginate($request->input('per_page', 10));

            return $notifications;
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }
    }


    public function markNotificationAsRead($id)
    {
        try {
            $user = Auth::user();

            if (!$user) {
                throw new \Exception("Unauthorized.");
            }

            // Accessing notifications directly through the user model
            $notification = $user->notifications()->find($id);

            if (!$notification) {
                throw new \Exception("Notification not found.");
            }

            if ($notification->read_at) {
                // If already marked as read, toggle its status to mark it as unread
                $notification->update(['read_at' => null]);
                return "Notification marked as unread successfully.";
            } else {
                // If not marked as read, mark it as read
                $notification->markAsRead();
                return "Notification marked as read successfully.";
            }
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }
    }



    public function markAllNotificationsAsRead()
    {
        try {
            $user = Auth::user();

            if (!$user) {
                throw new \Exception("Unauthorized."); // Unauthorized status code
            }

            // Accessing unread notifications directly through the user model and updating their read_at timestamps
            $user->unreadNotifications()->update(['read_at' => now()]);



            return "All notifications marked as read successfully.";
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }
    }
}
