<?php

use Illuminate\Http\Request;
use Illuminate\Support\Facades\Route;

use App\Http\Controllers\Admin\AuthController as AdminAuthController;
use App\Http\Controllers\Admin\PageController;
use App\Http\Controllers\Admin\PushNotificationController;
use App\Http\Controllers\Admin\UserController;
use App\Http\Controllers\NotificationController;

use App\Http\Controllers\Api\AuthController;
use App\Http\Controllers\Api\FeedbackController as ApiFeedbackController;
use App\Http\Controllers\Api\CountryController;
use App\Http\Controllers\Api\StateController;
use App\Http\Controllers\Api\CityController;

/*
|--------------------------------------------------------------------------
| API Routes
|--------------------------------------------------------------------------
|
| Here is where you can register API routes for your application. These
| routes are loaded by the RouteServiceProvider and all of them will
| be assigned to the "api" middleware group. Make something great!
|
*/

Route::middleware('auth:sanctum')->get('/user', function (Request $request) {
    return $request->user();
});

Route::prefix('admin')->group(function () {
    Route::post('/login', [AdminAuthController::class, 'login']);
    Route::post('/reset/password', [AdminAuthController::class, 'reset']);
    Route::post('/verify/code', [AdminAuthController::class, 'verifyCode']);
    Route::post('/set/password', [AdminAuthController::class, 'setPassword']);
    Route::post('/logout', [AdminAuthController::class, 'logout']);



    Route::middleware('auth:sanctum')->group(function () {
        // Admin Profile
        Route::get('profile', [AdminAuthController::class, 'profile']);
        Route::post('profile/update', [AdminAuthController::class, 'update']);
        Route::post('change-password', [AdminAuthController::class, 'updatePassword']);

        // Dashboard
        Route::get('dashboard/stats', [AdminAuthController::class, 'dashboardStats']);
        Route::get('dashboard/graph', [AdminAuthController::class, 'dashboardGraph']);
        Route::get('chart/data', [AdminAuthController::class, 'chartData']);
        Route::get('chart/filter', [AdminAuthController::class, 'filterChartData']);

        // User or Vendor
        Route::post('users', [UserController::class, 'index']);
        Route::post('user/{user}/status', [UserController::class, 'status']);
        Route::get('user/{user}', [UserController::class, 'show']);

        // Vendor
        Route::get('vendors', [UserController::class, 'index']);
        Route::post('vendor/{vendor}/status', [UserController::class, 'status']);
        Route::get('vendor/{vendor}', [UserController::class, 'show']);

        // Push Notification
        Route::post('send/push-notification', [PushNotificationController::class, 'send']);
        Route::get('get/push-notification', [PushNotificationController::class, 'index']);
        Route::get('get/push-notification/{push_notification}', [PushNotificationController::class, 'show']);


        // Pages
        Route::get('pages', [PageController::class, 'index']);
        Route::get('page/{page}', [PageController::class, 'show']);
        Route::post('page/{page}/update', [PageController::class, 'update']);

        // Notification
        Route::get('notification/unread/list', [NotificationController::class, 'unread']);
        Route::get('notification/all/list', [NotificationController::class, 'index']);
        Route::post('notification/all/read', [NotificationController::class, 'mark']);
        Route::post('notification/read', [NotificationController::class, 'individual']);
        Route::post('notification/unread', [NotificationController::class, 'markAsUnread']);
        Route::get('unread/notification/count', [NotificationController::class, 'unreadNotificationCount']);
        Route::get('unread/message/count', [NotificationController::class, 'unreadMessageCount']);
    });
});

// User APIs
Route::prefix('user')->group(function () {

    Route::post('/signup', [AuthController::class, 'signup']);
    Route::post('/login', [AuthController::class, 'login']);
    Route::post('/reset/password', [AuthController::class, 'reset']);
    Route::post('/verify/code', [AuthController::class, 'verifyCode']);
    Route::post('/set/password', [AuthController::class, 'setPassword']);
    Route::post('/logout', [AuthController::class, 'logout']);



    Route::middleware('auth:sanctum')->group(function () {
        // User Profile
        Route::get('profile', [AuthController::class, 'profile']);
        Route::post('profile/update', [AuthController::class, 'update']);
        Route::post('change-password', [AuthController::class, 'updatePassword']);


        //General
        Route::get('countries', [CountryController::class, 'index']);
        Route::get('states', [StateController::class, 'index']);
        Route::get('cities', [CityController::class, 'index']);


        // Page
        Route::get('page/{page}', [PageController::class, 'showBySlug']);

        // Feedback
        Route::post('contact-us', [ApiFeedbackController::class, 'store']);

        //Notification
        Route::get('notification/unread/list', [NotificationController::class, 'unread']);
        Route::get('notification/all/list', [NotificationController::class, 'index']);
        Route::post('notification/all/read', [NotificationController::class, 'mark']);
        Route::post('notification/read', [NotificationController::class, 'individual']);
        Route::post('notification/unread', [NotificationController::class, 'markAsUnread']);
        Route::get('unread/notification/count', [NotificationController::class, 'unreadNotificationCount']);
        Route::get('unread/message/count', [NotificationController::class, 'unreadMessageCount']);
    });
});
